@@ -16,11 +16,10 @@
 # limitations under the License.
 
 import collections
-import inspect
+import functools
 import logging as python_logging
 import select
 import time
-import traceback
 
 from exabgp.bgp import fsm as exa_fsm
 from exabgp.bgp import message as exa_message
@@ -32,7 +31,6 @@ from exabgp import reactor as exa_reactor
 from exabgp.reactor import peer as exa_peer
 from oslo_config import cfg
 from oslo_log import log as logging
-import six
 
 from networking_bagpipe.bagpipe_bgp.common import looking_glass as lg
 from networking_bagpipe.bagpipe_bgp import engine
@@ -58,7 +56,7 @@ def setup_exabgp_env():
     exa_logger.Logger._restart = exa_logger.Logger.restart
 
     def decorated_restart(f):
-        @six.wraps(f)
+        @functools.wraps(f)
         def restart_never_first(self, first):
             # we don't want exabgp to really ever do its first restart stuff
             # that resets the root logger handlers
@@ -79,21 +77,10 @@ def setup_exabgp_env():
     exa_logger.Logger._syslog.addHandler = noop
     exa_logger.Logger._syslog.removeHandler = noop
 
-    # patch the exabgp logging method, to avoid formating all the
-    # information twice
-    # we have to use different signature for exabgp (change betweeen
-    # 4.0.1 and 4.0.2)
-    FORMAT_SIG_401 = ['timestamp', 'level', 'source', 'message']
-    if inspect.getargspec(exa_logger.Logger._format).args == FORMAT_SIG_401:
-        def patched_format(self, timestamp, level, source, message):
-            if self.short:
-                return message
-            return "%-13s %s" % (source, message)
-    else:  # 4.0.2 or higher
-        def patched_format(self, message, source, level, timestamp=None):
-            if self.short:
-                return message
-            return "%-13s %s" % (source, message)
+    def patched_format(self, message, source, level, timestamp=None):
+        if self.short:
+            return message
+        return "%-13s %s" % (source, message)
 
     exa_logger.Logger._format = patched_format
 
@@ -238,7 +225,7 @@ class ExaBGPPeerWorker(bgp_peer_worker.BGPPeerWorker, lg.LookingGlassMixin):
                 raise bgp_peer_worker.OpenWaitTimeout(str(e))
             else:
                 raise Exception("Notify received: %s" % e)
-        except exa_reactor.network.error.LostConnection as e:
+        except exa_reactor.network.error.LostConnection:
             raise
 
         # check the capabilities of the session just established...
@@ -290,7 +277,7 @@ class ExaBGPPeerWorker(bgp_peer_worker.BGPPeerWorker, lg.LookingGlassMixin):
             if not self.protocol.connection:
                 raise Exception("lost connection")
 
-            message = six.next(self.protocol.read_message())
+            message = next(self.protocol.read_message())
 
             if message.ID != exa_message.NOP.ID:
                 self.log.debug("protocol read message: %s", message)
@@ -370,11 +357,10 @@ class ExaBGPPeerWorker(bgp_peer_worker.BGPPeerWorker, lg.LookingGlassMixin):
         self.log.debug("Sending %d bytes on socket to peer %s",
                        len(data), self.peer_address)
         try:
-            for _ in self.protocol.connection.writer(data):
+            for _unused in self.protocol.connection.writer(data):
                 pass
-        except Exception as e:
-            self.log.error("Was not able to send data: %s", e)
-            self.log.warning("%s", traceback.format_exc())
+        except Exception:
+            self.log.exception("Was not able to send data")
 
     def _keep_alive_message_data(self):
         return exa_message.KeepAlive().message()
@@ -384,10 +370,9 @@ class ExaBGPPeerWorker(bgp_peer_worker.BGPPeerWorker, lg.LookingGlassMixin):
             r = exa_message.Update([event.route_entry.nlri],
                                    event.route_entry.attributes)
             return ''.join(r.messages(self.protocol.negotiated))
-        except Exception as e:
-            self.log.error("Exception while generating message for "
-                           "route %s: %s", r, e)
-            self.log.warning("%s", traceback.format_exc())
+        except Exception:
+            self.log.exception("Exception while generating message for "
+                               "route %s", r)
             return ''
 
     # Looking Glass ###############
