@@ -15,11 +15,10 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import queue
 import threading
-import traceback
 
 from oslo_log import log as logging
-from six.moves import queue
 
 from networking_bagpipe.bagpipe_bgp.common import looking_glass as lg
 from networking_bagpipe.bagpipe_bgp import engine
@@ -55,6 +54,8 @@ class Worker(engine.EventSource, lg.LookingGlassMixin):
         # private data for RouteTableManager
         self._rtm_matches = set()
 
+        self._was_stopped = False
+
         LOG.debug("Instantiated %s worker", self.name)
 
     def stop(self):
@@ -64,11 +65,18 @@ class Worker(engine.EventSource, lg.LookingGlassMixin):
         and indicate to the route table manager that this worker is stopped.
         Then call _stopped() to let a subclass implement any further work.
         """
+        if self._was_stopped:
+            LOG.debug("not running, nothing to do to stop")
+            return
         LOG.info("Stop worker %s", self)
-        self._please_stop.set()
-        self.enqueue(STOP_EVENT)
+        self.stop_event_loop()
         self._cleanup()
         self._stopped()
+        self._was_stopped = True
+
+    def stop_event_loop(self):
+        self._please_stop.set()
+        self.enqueue(STOP_EVENT)
 
     def _cleanup(self):
         self.rtm.enqueue(engine.WorkerCleanupEvent(self))
@@ -83,7 +91,6 @@ class Worker(engine.EventSource, lg.LookingGlassMixin):
         """Main loop where the worker consumes events."""
 
         while not self._please_stop.isSet():
-            # LOG.debug("%s worker waiting on queue",self.name )
             event = self._dequeue()
 
             if event == STOP_EVENT:
@@ -91,12 +98,11 @@ class Worker(engine.EventSource, lg.LookingGlassMixin):
                 self._please_stop.set()
                 break
 
-            # LOG.debug("%s worker calling _on_event for %s", self.name, event)
             try:
                 self._on_event(event)
-            except Exception as e:
-                LOG.error("Exception raised on subclass._on_event: %s", e)
-                LOG.error("%s", traceback.format_exc())
+            except Exception:
+                LOG.exception("Exception raised on subclass._on_event: %s",
+                              event)
 
     def run(self):
         self._event_queue_processor_loop()
@@ -116,14 +122,20 @@ class Worker(engine.EventSource, lg.LookingGlassMixin):
         self._queue.put(event)
 
     def _subscribe(self, afi, safi, rt=None):
-        subobj = engine.Subscription(afi, safi, rt, self)
-        LOG.info("Subscribe: %s ", subobj)
-        self.rtm.enqueue(subobj)
+        try:
+            subobj = engine.Subscription(afi, safi, rt, self)
+            LOG.info("Subscribe: %s ", subobj)
+            self.rtm.enqueue(subobj)
+        except engine.UnsupportedRT as e:
+            LOG.debug("unsupported RT, ignoring (%s)", e.rt)
 
     def _unsubscribe(self, afi, safi, rt=None):
-        subobj = engine.Unsubscription(afi, safi, rt, self)
-        LOG.info("Unsubscribe: %s ", subobj)
-        self.rtm.enqueue(subobj)
+        try:
+            subobj = engine.Unsubscription(afi, safi, rt, self)
+            LOG.info("Unsubscribe: %s ", subobj)
+            self.rtm.enqueue(subobj)
+        except engine.UnsupportedRT as e:
+            LOG.debug("unsupported RT, ignoring (%s)", e.rt)
 
     def get_subscriptions(self):
         return sorted(self._rtm_matches)
