@@ -18,7 +18,6 @@
 import threading
 
 from oslo_log import log as logging
-import six
 
 from networking_bagpipe.bagpipe_bgp.common import exceptions as exc
 from networking_bagpipe.bagpipe_bgp.common import log_decorator
@@ -29,9 +28,8 @@ from networking_bagpipe.bagpipe_bgp import constants
 from networking_bagpipe.bagpipe_bgp.engine import bgp_manager
 from networking_bagpipe.bagpipe_bgp.vpn import dataplane_drivers as dp_drivers
 from networking_bagpipe.bagpipe_bgp.vpn import evpn
+from networking_bagpipe.bagpipe_bgp.vpn import identifier_allocators
 from networking_bagpipe.bagpipe_bgp.vpn import ipvpn
-from networking_bagpipe.bagpipe_bgp.vpn import label_allocator
-from networking_bagpipe.bagpipe_bgp.vpn import rd_allocator
 
 from neutron_lib import exceptions
 
@@ -50,7 +48,7 @@ class MaxInstanceIDReached(exceptions.NeutronException):
     _message = "Could not create VPN instance: max instance id was reached"
 
 
-class VPNManager(lg.LookingGlassMixin):
+class VPNManager(lg.LookingGlassMixin, utils.ClassReprMixin):
     """VPN Manager
 
     Creates, and keeps track of, VPN instances (VRFs and EVIs) and passes
@@ -67,7 +65,7 @@ class VPNManager(lg.LookingGlassMixin):
     def __init__(self):
         LOG.debug("Instantiating VPN Manager...")
 
-        self.bgp_manager = bgp_manager.Manager.get_instance()
+        self.bgp_manager = bgp_manager.Manager()
 
         self.dataplane_drivers = dp_drivers.instantiate_dataplane_drivers()
 
@@ -76,17 +74,21 @@ class VPNManager(lg.LookingGlassMixin):
         self.next_vpn_instance_id = 1
 
         LOG.debug("Creating label allocator")
-        self.label_allocator = label_allocator.LabelAllocator()
+        self.label_allocator = identifier_allocators.LabelAllocator()
 
         LOG.debug("Creating route distinguisher allocator")
-        self.rd_allocator = rd_allocator.RDAllocator(
+        self.rd_allocator = identifier_allocators.RDAllocator(
             self.bgp_manager.get_local_address())
 
         # dict containing info how an ipvpn is plugged
         # from an evpn  (keys: ipvpn instances)
         self._evpn_ipvpn_ifs = {}
 
-        self.lock = threading.Lock()
+        # keys: vni
+        # value: VPNInstance
+        self.vpn_instance_by_vni = {}
+
+        self.lock = threading.RLock()
 
     def _run_command(self, *args, **kwargs):
         run_command.run_command(LOG, *args, run_as_root=True, **kwargs)
@@ -223,6 +225,8 @@ class VPNManager(lg.LookingGlassMixin):
         # if one already exists, check matching instance_type
         # else create one with provided parameters and start it
         #   (unless create_if_none is False --> raise exc.VPNNotFound)
+        # returns True if an already started instance was found
+        # False if a new instance was created without starting it
 
         LOG.info("Finding %s for external vpn_instance identifier %s",
                  instance_type, external_instance_id)
@@ -236,11 +240,16 @@ class VPNManager(lg.LookingGlassMixin):
                                 "(asked %s vs. already having %s)"
                                 % (external_instance_id,
                                    instance_type, vpn_instance.type))
-            return vpn_instance
+            return vpn_instance, True
 
         if not kwargs.pop('create_if_none', True):
             raise exc.VPNNotFound(external_instance_id)
 
+        # if a vni is specified, check that no VPN instance with same VNI
+        # already exists...
+        if 'vni' in kwargs and kwargs['vni'] in self.vpn_instance_by_vni:
+            raise exc.APIAlreadyUsedVNI(kwargs['vni'])
+
         vpn_instance_class = VPNManager.type2class[instance_type]
         dataplane_driver = self.dataplane_drivers[instance_type]
 
@@ -258,16 +267,24 @@ class VPNManager(lg.LookingGlassMixin):
                                           readvertise, attract_traffic,
                                           fallback, **kwargs)
 
-        self.vpn_instances[external_instance_id] = vpn_instance
+        self.register_vpn_instance(vpn_instance)
 
-        vpn_instance.start()
-
-        return vpn_instance
+        return vpn_instance, False
 
     @utils.synchronized
     @log_decorator.log_info
-    def remove_from_vpn_instances(self, external_instance_id):
-        del self.vpn_instances[external_instance_id]
+    def register_vpn_instance(self, vpn_instance):
+        self.vpn_instances[vpn_instance.external_instance_id] = vpn_instance
+        if vpn_instance.forced_vni:
+            self.vpn_instance_by_vni[
+                vpn_instance.instance_label] = vpn_instance
+
+    @utils.synchronized
+    @log_decorator.log_info
+    def unregister_vpn_instance(self, vpn_instance):
+        del self.vpn_instances[vpn_instance.external_instance_id]
+        if vpn_instance.forced_vni:
+            del self.vpn_instance_by_vni[vpn_instance.instance_label]
 
     def _check_instance_type(self, params):
         if 'vpn_type' not in params:
@@ -308,6 +325,7 @@ class VPNManager(lg.LookingGlassMixin):
         vni = params.get('vni')
 
         ip_address_prefix = params.get('ip_address_prefix')
+        ip_address_plen = params.get('ip_address_plen')
 
         # Convert route target string to RouteTarget dictionary
         import_rts = utils.convert_route_targets(import_rts)
@@ -329,26 +347,20 @@ class VPNManager(lg.LookingGlassMixin):
             except KeyError as e:
                 raise Exception("Wrong 'attract_traffic' parameters: %s" % e)
 
-        # retrieve network mask
-        mask = int(ip_address_prefix.split('/')[1])
-
         kwargs = {}
         if vni:
             kwargs['vni'] = vni
         if instance_type == constants.EVPN and linuxbr:
             kwargs['linuxbr'] = linuxbr
 
-        vpn_instance = self._get_vpn_instance(
+        vpn_instance, started = self._get_vpn_instance(
             external_instance_id, instance_type, import_rts, export_rts,
-            gateway_ip, mask, readvertise, attract_traffic, fallback, **kwargs)
+            gateway_ip, ip_address_plen, readvertise, attract_traffic,
+            fallback, **kwargs)
 
         vpn_instance.description = params.get('instance_description')
 
-        # Check if new route target import/export must be updated
-        if not ((set(vpn_instance.import_rts) == set(import_rts)) and
-                (set(vpn_instance.export_rts) == set(export_rts))):
-            vpn_instance.update_route_targets(import_rts, export_rts)
-
+        vpn_instance.update_route_targets(import_rts, export_rts)
         vpn_instance.update_fallback(fallback)
 
         if instance_type == constants.IPVPN and 'evpn' in localport:
@@ -366,6 +378,12 @@ class VPNManager(lg.LookingGlassMixin):
                                  advertise_subnet, lb_consistent_hash_order,
                                  local_pref, **plug_kwargs)
 
+        # delaying the start after the first vif_plugged allows to handle
+        # dataplane driver for which the first vif_plugged needs to happen
+        # before route advertisements can be processed
+        if not started:
+            vpn_instance.start()
+
     @log_decorator.log_info
     def unplug_vif_from_vpn(self, **params):
 
@@ -394,14 +412,14 @@ class VPNManager(lg.LookingGlassMixin):
             self._detach_evpn_2_ipvpn(vpn_instance)
 
         if vpn_instance.stop_if_empty():
-            self.remove_from_vpn_instances(external_instance_id)
+            self.unregister_vpn_instance(vpn_instance)
 
     def redirect_instance_for_rt(self, redirected_type, redirect_rt,
                                  stop=False):
         external_instance_id = redirect_instance_extid(redirected_type,
                                                        redirect_rt)
-        LOG.info("Need VPN instance %s for traffic redirection to route "
-                 "target %s", external_instance_id, redirect_rt)
+        LOG.info("Need VPN instance %s for traffic redirection to RT %s",
+                 external_instance_id, redirect_rt)
 
         # Convert route target string to RouteTarget dictionary
         import_rts = utils.convert_route_targets([redirect_rt])
@@ -409,12 +427,15 @@ class VPNManager(lg.LookingGlassMixin):
         # Retrieve a redirect VPN instance or create a new one if none exists
         # yet
         try:
-            return self._get_vpn_instance(external_instance_id,
+            i, s = self._get_vpn_instance(external_instance_id,
                                           redirected_type,
                                           import_rts, [],
                                           "127.0.0.1",
                                           "24", None, None,
                                           create_if_none=(not stop))
+            if not s:
+                i.start()
+            return i
         except exc.VPNNotFound:
             # (reached only in the 'stop' case)
             LOG.error("Try to stop traffic redirection for an RT for which"
@@ -436,21 +457,19 @@ class VPNManager(lg.LookingGlassMixin):
                                                           redirect_rt,
                                                           stop=True)
         redirect_instance.unregister_redirected_instance(redirected_id)
-
         if redirect_instance.stop_if_no_redirected_instance():
-            self.remove_from_vpn_instances(
-                redirect_instance.external_instance_id)
+            self.unregister_vpn_instance(redirect_instance)
 
     @log_decorator.log_info
     def stop(self):
         self.bgp_manager.stop()
-        for vpn_instance in six.itervalues(self.vpn_instances):
+        for vpn_instance in self.vpn_instances.values():
             vpn_instance.stop()
             # Cleanup veth pair
             if (vpn_instance.type == constants.IPVPN and
                     self._evpn_ipvpn_ifs.get(vpn_instance)):
                 self._cleanup_evpn2ipvpn(vpn_instance)
-        for vpn_instance in six.itervalues(self.vpn_instances):
+        for vpn_instance in self.vpn_instances.values():
             vpn_instance.join()
         self.vpn_instances.clear()
 
@@ -494,7 +513,8 @@ class VPNManager(lg.LookingGlassMixin):
         return {
             "instances": (lg.COLLECTION, (self.get_lg_vpn_list,
                                           self.get_lg_vpn_from_path_item)),
-            "dataplane": (lg.DELEGATE, dataplane_hook)
+            "dataplane": (lg.DELEGATE, dataplane_hook),
+            "instances_per_vni": (lg.SUBITEM, self.get_lg_instances_per_vni)
         }
 
     def get_lg_vpn_list(self):
@@ -507,6 +527,12 @@ class VPNManager(lg.LookingGlassMixin):
     def get_vpn_instances_count(self):
         return len(self.vpn_instances)
 
+    def get_lg_instances_per_vni(self):
+        return {vni: {'name': str(instance),
+                      'external_instance_id': instance.external_instance_id
+                      }
+                for vni, instance in self.vpn_instance_by_vni.items()}
+
     def get_lg_dataplanes_list(self):
         return [{"id": i} for i in self.dataplane_drivers.keys()]
 
