@@ -18,9 +18,9 @@ from netaddr.ip import IPNetwork
 
 from neutron_lib.api.definitions import bgpvpn as bgpvpn_def
 from neutron_lib.api.definitions import provider_net as pnet
+from neutron_lib.db import model_query
 from neutron_lib.plugins import directory
 
-from oslo_config import cfg
 from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
@@ -41,18 +41,6 @@ from networking_sfc.services.sfc.drivers import base as driver_base
 
 LOG = logging.getLogger(__name__)
 
-sfc_bagpipe_opts = [
-    cfg.IntOpt('as_number', default=64512,
-               help=_("Autonomous System number.")),
-    cfg.ListOpt('rtnn',
-                default=[5000, 5999],
-                help=_("List containing [<rtnn_min>, <rtnn_max>] values "
-                       "enumerating ranges of Route Target number that are "
-                       "available for Port Chain allocation")),
-]
-
-cfg.CONF.register_opts(sfc_bagpipe_opts, "sfc_bagpipe")
-
 
 class BaGPipeSfcDriver(driver_base.SfcDriverBase,
                        sfc_db.BaGPipeSfcDriverDB):
@@ -60,7 +48,7 @@ class BaGPipeSfcDriver(driver_base.SfcDriverBase,
 
     def initialize(self):
         super(BaGPipeSfcDriver, self).initialize()
-        self.rt_allocator = sfc_db.RTAllocator(cfg.CONF.sfc_bagpipe)
+        self.rt_allocator = sfc_db.RTAllocator()
         self._push_rpc = resources_rpc.ResourcesPushRpcApi()
 
     def _parse_ipaddress_prefix(self, cidr):
@@ -120,7 +108,7 @@ class BaGPipeSfcDriver(driver_base.SfcDriverBase,
             return bgpvpns
 
         bgpvpn_plugin = (
-            directory.get_plugin(bgpvpn_def.LABEL)
+            directory.get_plugin(bgpvpn_def.ALIAS)
         )
         if not bgpvpn_plugin:
             LOG.warning("BGPVPN service not found")
@@ -143,8 +131,9 @@ class BaGPipeSfcDriver(driver_base.SfcDriverBase,
         core_plugin = directory.get_plugin()
         network = core_plugin.get_network(self.admin_context, network_id)
 
-        return ':'.join([str(cfg.CONF.sfc_bagpipe.as_number),
-                         str(network[pnet.SEGMENTATION_ID])])
+        return (
+            self.rt_allocator._get_rt_from_rtnn(network[pnet.SEGMENTATION_ID])
+        )
 
     def _get_bgpvpn_rts(self, bgpvpn_list):
         import_rts = set()
@@ -199,7 +188,7 @@ class BaGPipeSfcDriver(driver_base.SfcDriverBase,
 
             logical_port = fc['logical_' + side + '_port']
             if logical_port is not None:
-                port = self._get_by_id(
+                port = model_query.get_by_id(
                     self.admin_context, models_v2.Port, logical_port
                 )
                 if fc[side + '_ip_prefix'] is None:
@@ -407,30 +396,54 @@ class BaGPipeSfcDriver(driver_base.SfcDriverBase,
                     is_redirect=True,
                     reverse=reverse)
 
-                prev_readv_from_rts = ((src_rts if reverse else dest_rts)
-                                       if egress_bgpvpns else None)
-                prev_readv_to_rt = (prev_redirect_rt
-                                    if egress_bgpvpns else None)
-                prev_attract_to_rt = (prev_redirect_rt
-                                      if not egress_bgpvpns else None)
+                if position+1 == len(reversed_ppg)-1:
+                    # Advertise FlowSpec routes from last intermediate hop
+                    prev_readv_from_rts = ((src_rts if reverse else dest_rts)
+                                           if egress_bgpvpns else None)
+                    prev_readv_to_rt = (prev_redirect_rt
+                                        if egress_bgpvpns else None)
+                    prev_attract_to_rt = (prev_redirect_rt
+                                          if not egress_bgpvpns else None)
 
-                hop_detail_obj = sfc_obj.BaGPipeChainHop(
-                    context._plugin_context,
-                    id=uuidutils.generate_uuid(),
-                    project_id=project_id,
-                    portchain_id=port_chain['id'],
-                    rts=[prev_ppg_rt],
-                    ingress_gw=current_subnet['gateway_ip'],
-                    egress_gw=prev_subnet['gateway_ip'],
-                    reverse_hop=reverse,
-                    ingress_ppg=current_ppg['id'],
-                    egress_ppg=prev_ppg['id'],
-                    readv_from_rts=prev_readv_from_rts,
-                    readv_to_rt=prev_readv_to_rt,
-                    attract_to_rt=prev_attract_to_rt,
-                    redirect_rts=[prev_ppg_rt],
-                    classifiers=jsonutils.dumps(classifiers)
-                )
+                    hop_detail_obj = sfc_obj.BaGPipeChainHop(
+                        context._plugin_context,
+                        id=uuidutils.generate_uuid(),
+                        project_id=project_id,
+                        portchain_id=port_chain['id'],
+                        rts=[prev_ppg_rt],
+                        ingress_gw=current_subnet['gateway_ip'],
+                        egress_gw=prev_subnet['gateway_ip'],
+                        reverse_hop=reverse,
+                        ingress_ppg=current_ppg['id'],
+                        egress_ppg=prev_ppg['id'],
+                        readv_from_rts=prev_readv_from_rts,
+                        readv_to_rt=prev_readv_to_rt,
+                        attract_to_rt=prev_attract_to_rt,
+                        redirect_rts=[prev_ppg_rt],
+                        classifiers=jsonutils.dumps(classifiers)
+                    )
+                else:
+                    # Readvertise FlowSpec routes between intermediate hops
+                    from_redirect_rt = (
+                        self.rt_allocator.get_redirect_rt_by_ppg(
+                            reversed_ppg[position+2]))
+
+                    hop_detail_obj = sfc_obj.BaGPipeChainHop(
+                        context._plugin_context,
+                        id=uuidutils.generate_uuid(),
+                        project_id=project_id,
+                        portchain_id=port_chain['id'],
+                        rts=[prev_ppg_rt],
+                        ingress_gw=current_subnet['gateway_ip'],
+                        egress_gw=prev_subnet['gateway_ip'],
+                        reverse_hop=reverse,
+                        ingress_ppg=current_ppg['id'],
+                        egress_ppg=prev_ppg['id'],
+                        readv_from_rts=[from_redirect_rt],
+                        readv_to_rt=prev_redirect_rt,
+                        redirect_rts=[prev_ppg_rt],
+                        classifiers=jsonutils.dumps(classifiers)
+                    )
                 hop_detail_obj.create()
                 hop_details.append(hop_detail_obj)
 
@@ -456,34 +469,61 @@ class BaGPipeSfcDriver(driver_base.SfcDriverBase,
                     is_redirect=True,
                     reverse=reverse)
 
-                first_readv_from_rts = ((src_rts if reverse else dest_rts)
-                                        if egress_bgpvpns else None)
-                first_readv_to_rt = (first_redirect_rt
-                                     if egress_bgpvpns else None)
-                first_attract_to_rt = (first_redirect_rt
-                                       if not egress_bgpvpns else None)
                 first_rts = ((dest_rts if reverse else src_rts)
                              if ingress_bgpvpns else [first_ppg_rt])
 
-                hop_detail_obj = sfc_obj.BaGPipeChainHop(
-                    context._plugin_context,
-                    id=uuidutils.generate_uuid(),
-                    project_id=project_id,
-                    portchain_id=port_chain['id'],
-                    rts=first_rts,
-                    ingress_gw=(dest_subnet['gateway_ip'] if reverse
-                                else src_subnet['gateway_ip']),
-                    egress_gw=first_subnet['gateway_ip'],
-                    reverse_hop=reverse,
-                    ingress_network=(dest_subnet['network_id'] if reverse
-                                     else src_subnet['network_id']),
-                    egress_ppg=first_ppg['id'],
-                    readv_from_rts=first_readv_from_rts,
-                    readv_to_rts=first_readv_to_rt,
-                    attract_to_rt=first_attract_to_rt,
-                    redirect_rts=first_rts,
-                    classifiers=jsonutils.dumps(classifiers)
-                )
+                if len(reversed_ppg) == 1:
+                    first_readv_from_rts = ((src_rts if reverse else dest_rts)
+                                            if egress_bgpvpns else None)
+                    first_readv_to_rt = (first_redirect_rt
+                                         if egress_bgpvpns else None)
+                    first_attract_to_rt = (first_redirect_rt
+                                           if not egress_bgpvpns else None)
+                    first_rts = ((dest_rts if reverse else src_rts)
+                                 if ingress_bgpvpns else [first_ppg_rt])
+
+                    hop_detail_obj = sfc_obj.BaGPipeChainHop(
+                        context._plugin_context,
+                        id=uuidutils.generate_uuid(),
+                        project_id=project_id,
+                        portchain_id=port_chain['id'],
+                        rts=first_rts,
+                        ingress_gw=(dest_subnet['gateway_ip'] if reverse
+                                    else src_subnet['gateway_ip']),
+                        egress_gw=first_subnet['gateway_ip'],
+                        reverse_hop=reverse,
+                        ingress_network=(dest_subnet['network_id'] if reverse
+                                         else src_subnet['network_id']),
+                        egress_ppg=first_ppg['id'],
+                        readv_from_rts=first_readv_from_rts,
+                        readv_to_rt=first_readv_to_rt,
+                        attract_to_rt=first_attract_to_rt,
+                        redirect_rts=first_rts,
+                        classifiers=jsonutils.dumps(classifiers)
+                    )
+                else:
+                    from_redirect_rt = (
+                        self.rt_allocator.get_redirect_rt_by_ppg(
+                            reversed_ppg[position+1]))
+
+                    hop_detail_obj = sfc_obj.BaGPipeChainHop(
+                        context._plugin_context,
+                        id=uuidutils.generate_uuid(),
+                        project_id=project_id,
+                        portchain_id=port_chain['id'],
+                        rts=first_rts,
+                        ingress_gw=(dest_subnet['gateway_ip'] if reverse
+                                    else src_subnet['gateway_ip']),
+                        egress_gw=first_subnet['gateway_ip'],
+                        reverse_hop=reverse,
+                        ingress_network=(dest_subnet['network_id'] if reverse
+                                         else src_subnet['network_id']),
+                        egress_ppg=first_ppg['id'],
+                        readv_from_rts=[from_redirect_rt],
+                        readv_to_rt=first_redirect_rt,
+                        redirect_rts=first_rts,
+                        classifiers=jsonutils.dumps(classifiers)
+                    )
                 hop_detail_obj.create()
                 hop_details.append(hop_detail_obj)
 
