@@ -15,8 +15,9 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
+from unittest import mock
 
+from networking_bagpipe.bagpipe_bgp.common import dataplane_utils
 from networking_bagpipe.bagpipe_bgp.vpn import dataplane_drivers as dp_drivers
 from networking_bagpipe.bagpipe_bgp.vpn.evpn import ovs
 from networking_bagpipe.tests.unit.bagpipe_bgp import base as t
@@ -24,7 +25,7 @@ from networking_bagpipe.tests.unit.bagpipe_bgp import base as t
 from neutron.agent.common import ovs_lib
 from neutron.plugins.ml2.drivers.openvswitch.agent.common import \
     constants as ovs_const
-from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl import \
+from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native import \
     br_tun
 
 LOCAL_IP = "1.2.3.4"
@@ -41,7 +42,7 @@ class TestTunnelManager(t.TestCase):
         self.manager = ovs.TunnelManager(self.bridge, LOCAL_IP)
 
     def test_get_tunnel(self):
-        t1 = self.manager.tunnel_for_remote_ip("2.2.2.2", "A")
+        t1, _ = self.manager.get_object("2.2.2.2", "A")
         self.bridge.add_tunnel_port.assert_called_once_with(mock.ANY,
                                                             "2.2.2.2",
                                                             LOCAL_IP,
@@ -51,7 +52,7 @@ class TestTunnelManager(t.TestCase):
         self.bridge.add_tunnel_port.reset_mock()
         self.bridge.setup_tunnel_port.reset_mock()
 
-        t2 = self.manager.tunnel_for_remote_ip("2.2.2.2", "B")
+        t2, _ = self.manager.get_object("2.2.2.2", "B")
         self.bridge.add_tunnel_port.assert_not_called()
         self.bridge.setup_tunnel_port.assert_not_called()
 
@@ -60,7 +61,7 @@ class TestTunnelManager(t.TestCase):
         self.bridge.add_tunnel_port.reset_mock()
         self.bridge.setup_tunnel_port.reset_mock()
 
-        t3 = self.manager.tunnel_for_remote_ip("3.3.3.3", "A")
+        t3, _ = self.manager.get_object("3.3.3.3", "A")
         self.bridge.add_tunnel_port.assert_called_once()
         self.bridge.setup_tunnel_port.assert_called_once()
 
@@ -68,36 +69,36 @@ class TestTunnelManager(t.TestCase):
 
         self.assertTrue(len(self.manager.infos()))
 
-    def test_free_tunnel(self):
-        t1 = self.manager.tunnel_for_remote_ip("2.2.2.2", "A")
-        self.manager.tunnel_for_remote_ip("2.2.2.2", "B")
-        t2 = self.manager.tunnel_for_remote_ip("3.3.3.3", "A")
+    def test_free_object(self):
+        t1, _ = self.manager.get_object("2.2.2.2", "A")
+        self.manager.get_object("2.2.2.2", "B")
+        t2, _ = self.manager.get_object("3.3.3.3", "A")
 
         self.bridge.add_tunnel_port.reset_mock()
         self.bridge.delete_port.reset_mock()
         self.bridge.setup_tunnel_port.reset_mock()
 
-        self.manager.free_tunnel("2.2.2.2", "A")
+        self.manager.free_object("2.2.2.2", "A")
         self.bridge.delete_port.assert_not_called()
-        t1bis = self.manager.tunnel_for_remote_ip("2.2.2.2")
+        t1bis = self.manager.find_object("2.2.2.2")
         self.assertTrue(t1bis == t1)
 
         self.bridge.add_tunnel_port.reset_mock()
         self.bridge.delete_port.reset_mock()
         self.bridge.setup_tunnel_port.reset_mock()
 
-        self.manager.free_tunnel("2.2.2.2", "B")
+        self.manager.free_object("2.2.2.2", "B")
         self.bridge.delete_port.assert_called_once_with(t1)
 
         self.bridge.add_tunnel_port.reset_mock()
         self.bridge.delete_port.reset_mock()
         self.bridge.setup_tunnel_port.reset_mock()
 
-        self.manager.free_tunnel("3.3.3.3", "A")
+        self.manager.free_object("3.3.3.3", "A")
         self.bridge.delete_port.assert_called_once_with(t2)
 
 
-class FackBridgeMockSpec(ovs.OVSBridgeWithGroups,
+class FakeBridgeMockSpec(dataplane_utils.OVSBridgeWithGroups,
                          br_tun.OVSTunnelBridge,
                          ovs_lib.OVSBridge):
     pass
@@ -114,8 +115,10 @@ class TestOVSEVIDataplane(t.TestCase):
     def setUp(self):
         super(TestOVSEVIDataplane, self).setUp()
 
-        self.bridge = mock.Mock(spec=FackBridgeMockSpec)
-        self.tunnel_mgr = mock.Mock(ovs.TunnelManager)
+        self.bridge = mock.Mock(spec=FakeBridgeMockSpec)
+        self.tunnel_mgr = mock.Mock(spec=ovs.TunnelManager)
+        self.tunnel_mgr.get_object.return_value = ("TUNNEL1", None)
+        self.tunnel_mgr.find_object.return_value = "TUNNEL1"
 
         self.dp_driver = mock.Mock(spec=dp_drivers.DataplaneDriver)
         self.dp_driver.bridge = self.bridge
@@ -141,24 +144,29 @@ class TestOVSEVIDataplane(t.TestCase):
         self.dataplane.setup_dataplane_for_remote_endpoint(
             MAC1, LOCAL_IP, 42, FakeNLRI("11.0.0.1"), None)
 
-        self.tunnel_mgr.tunnel_for_remote_ip.assert_not_called()
+        self.tunnel_mgr.get_object.assert_not_called()
 
     def test_setup_dataplane_for_remote_endpoint(self):
         self.dataplane.setup_dataplane_for_remote_endpoint(
             MAC1, "2.2.2.2", 42, FakeNLRI("11.0.0.1"), None)
 
-        self.tunnel_mgr.tunnel_for_remote_ip.assert_called_once_with(
-            "2.2.2.2", (77, 42, MAC1))
+        self.tunnel_mgr.get_object.assert_called_once_with(
+            "2.2.2.2", (77, (42, MAC1)))
 
-        self.bridge.add_flow.assert_called()
+        self.bridge.add_flow.assert_called_with(
+            table=ovs_const.UCAST_TO_TUN,
+            priority=mock.ANY,
+            dl_vlan=self.vlan,
+            dl_dst=MAC1,
+            actions=mock.ANY)
 
     def test_remove_dataplane_for_remote_endpoint__local(self):
         self.dataplane.remove_dataplane_for_remote_endpoint(
-            MAC1, "2.2.2.2", 42, FakeNLRI("11.0.0.1"))
+            MAC1, LOCAL_IP, 42, FakeNLRI("11.0.0.1"))
 
         self.bridge.delete_unicast_to_tun.assert_called_with(self.vlan, MAC1)
 
-        self.tunnel_mgr.free_tunnel.test_assert_not_called()
+        self.tunnel_mgr.free_object.assert_not_called()
 
     def test_remove_dataplane_for_remote_endpoint(self):
         self.dataplane.remove_dataplane_for_remote_endpoint(
@@ -166,37 +174,37 @@ class TestOVSEVIDataplane(t.TestCase):
 
         self.bridge.delete_unicast_to_tun.assert_called_with(self.vlan, MAC1)
 
-        self.tunnel_mgr.free_tunnel.assert_called_with(
-            "2.2.2.2", (77, 42, MAC1))
+        self.tunnel_mgr.free_object.assert_called_with(
+            "2.2.2.2", (77, (42, MAC1)))
 
     def test_add_dataplane_for_bum_endpoint__local(self):
         self.dataplane.add_dataplane_for_bum_endpoint(LOCAL_IP, 45,
                                                       None, None)
 
-        self.tunnel_mgr.tunnel_for_remote_ip.assert_not_called()
+        self.tunnel_mgr.get_object.assert_not_called()
 
     def test_add_dataplane_for_bum_endpoint(self):
         self.dataplane.add_dataplane_for_bum_endpoint("2.2.2.2", 45,
                                                       None, None)
 
-        self.tunnel_mgr.tunnel_for_remote_ip.assert_called_with(
-            "2.2.2.2", (77, 45, "flood"))
+        self.tunnel_mgr.get_object.assert_called_with(
+            "2.2.2.2", (77, (45, "flood")))
 
     def test_remove_dataplane_for_bum_endpoint__local(self):
         self.dataplane.add_dataplane_for_bum_endpoint(LOCAL_IP, 45,
                                                       None, None)
-        self.tunnel_mgr.free_tunnel.reset_mock()
+        self.tunnel_mgr.free_object.reset_mock()
 
         self.dataplane.remove_dataplane_for_bum_endpoint(LOCAL_IP, 45, None)
 
-        self.tunnel_mgr.free_tunnel.assert_not_called()
+        self.tunnel_mgr.free_object.assert_not_called()
 
     def test_remove_dataplane_for_bum_endpoint(self):
         self.dataplane.add_dataplane_for_bum_endpoint("2.2.2.2", 45,
                                                       None, None)
-        self.tunnel_mgr.free_tunnel.reset_mock()
+        self.tunnel_mgr.free_object.reset_mock()
 
         self.dataplane.remove_dataplane_for_bum_endpoint("2.2.2.2", 45, None)
 
-        self.tunnel_mgr.free_tunnel.assert_called_with(
-            "2.2.2.2", (77, 45, "flood"))
+        self.tunnel_mgr.free_object.assert_called_with(
+            "2.2.2.2", (77, (45, "flood")))
